@@ -143,6 +143,30 @@ enum cublasLtMatmulTile_t {
     CUBLASLT_MATMUL_TILE_END
 };
 
+/** Pointer mode to use for alpha/beta */
+enum cublasLtPointerMode_t {
+    /** matches CUBLAS_POINTER_MODE_HOST, pointer targets a single value host memory */
+    CUBLASLT_POINTER_MODE_HOST = CUBLAS_POINTER_MODE_HOST,
+    /** matches CUBLAS_POINTER_MODE_DEVICE, pointer targets a single value device memory */
+    CUBLASLT_POINTER_MODE_DEVICE = CUBLAS_POINTER_MODE_DEVICE,
+    /** pointer targets an array in device memory */
+    CUBLASLT_POINTER_MODE_DEVICE_VECTOR = 2,
+    /** alpha pointer targets an array in device memory, beta is zero */
+    CUBLASLT_POINTER_MODE_ALPHA_DEVICE_VECTOR_BETA_ZERO = 3,
+};
+
+/** Mask to define and query pointer mode capability */
+enum cublasLtPointerModeMask_t {
+    /** see CUBLASLT_POINTER_MODE_HOST */
+    CUBLASLT_POINTER_MODE_MASK_HOST = 1,
+    /** see CUBLASLT_POINTER_MODE_DEVICE */
+    CUBLASLT_POINTER_MODE_MASK_DEVICE = 2,
+    /** see CUBLASLT_POINTER_MODE_DEVICE_VECTOR */
+    CUBLASLT_POINTER_MODE_MASK_DEVICE_VECTOR = 4,
+    /** see CUBLASLT_POINTER_MODE_ALPHA_DEVICE_VECTOR_BETA_ZERO */
+    CUBLASLT_POINTER_MODE_MASK_ALPHA_DEVICE_VECTOR_BETA_ZERO = 8,
+};
+
 /** Execute matrix multiplication (D = alpha * op(A) * op(B) + beta * C).
  *
  * \retval     CUBLAS_STATUS_NOT_INITIALIZED   if cuBLASLt handle has not been initialized
@@ -156,7 +180,7 @@ enum cublasLtMatmulTile_t {
  * \retval     CUBLAS_STATUS_SUCCESS           if the operation completed successfully
  */
 cublasStatus_t CUBLASWINAPI
-cublasLtMatmul(cublasLtHandle_t lightHandle,               
+cublasLtMatmul(cublasLtHandle_t lightHandle,
                cublasLtMatmulDesc_t computeDesc,
                const void *alpha, /* host or device pointer */
                const void *A,
@@ -360,8 +384,8 @@ cublasLtMatrixLayoutGetAttribute(  //
 
 /** Matmul descriptor attributes to define details of the operation. */
 typedef enum {
-    /** Compute type, see cudaDataType. Defines data type used for multiply and accumulate operations and the accumulator
-     * during matrix multiplication.
+    /** Compute type, see cudaDataType. Defines data type used for multiply and accumulate operations and the
+     * accumulator during matrix multiplication.
      *
      * int32_t
      */
@@ -375,37 +399,51 @@ typedef enum {
      */
     CUBLASLT_MATMUL_DESC_SCALE_TYPE,
 
-    /** Pointer mode of alpha and beta, see cublasPointerMode_t.
-     * 
-     * int32_t, default: CUBLAS_POINTER_MODE_HOST
+    /** Pointer mode of alpha and beta, see cublasLtPointerMode_t. When CUBLASLT_POINTER_MODE_DEVICE_VECTOR is in use,
+     * alpha/beta vector lenghts must match number of output matrix rows.
+     *
+     * int32_t, default: CUBLASLT_POINTER_MODE_HOST
      */
     CUBLASLT_MATMUL_DESC_POINTER_MODE,
 
     /** Transform of matrix A, see cublasOperation_t.
-     * 
+     *
      * int32_t, default: CUBLAS_OP_N
      */
     CUBLASLT_MATMUL_DESC_TRANSA,
 
     /** Transform of matrix B, see cublasOperation_t.
-     * 
+     *
      * int32_t, default: CUBLAS_OP_N
      */
     CUBLASLT_MATMUL_DESC_TRANSB,
 
     /** Transform of matrix C, see cublasOperation_t.
-     * 
+     *
      * Must be CUBLAS_OP_N if performing matrix multiplication in place (when C == D).
-     * 
+     *
      * int32_t, default: CUBLAS_OP_N
      */
     CUBLASLT_MATMUL_DESC_TRANSC,
 
     /** Matrix fill mode, see cublasFillMode_t.
-     * 
+     *
      * int32_t, default: CUBLAS_FILL_MODE_FULL
      */
     CUBLASLT_MATMUL_DESC_FILL_MODE,
+
+    /** Epilogue function, see cublasLtEpilogue_t.
+     *
+     * uint32_t, default: CUBLASLT_EPILOGUE_DEFAULT
+     */
+    CUBLASLT_MATMUL_DESC_EPILOGUE,
+
+    /** Bias vector pointer in the device memory, see CUBLASLT_EPILOGUE_BIAS. Bias vector elements are the same type as
+     * alpha, beta (see CUBLASLT_MATMUL_DESC_SCALE_TYPE). Bias vector length must match matrix D rows count.
+     *
+     * const void *, default: NULL
+     */
+    CUBLASLT_MATMUL_DESC_BIAS_POINTER,
 } cublasLtMatmulDescAttributes_t;
 
 /** Create new matmul operation descriptor.
@@ -477,9 +515,9 @@ typedef enum {
      */
     CUBLASLT_MATRIX_TRANSFORM_DESC_SCALE_TYPE,
 
-    /** Pointer mode of alpha and beta, see cublasPointerMode_t.
+    /** Pointer mode of alpha and beta, see cublasLtPointerMode_t.
      * 
-     * int32_t, default: CUBLAS_POINTER_MODE_HOST
+     * int32_t, default: CUBLASLT_POINTER_MODE_HOST
      */
     CUBLASLT_MATRIX_TRANSFORM_DESC_POINTER_MODE,
 
@@ -581,12 +619,34 @@ typedef enum {
     CUBLASLT_REDUCTION_SCHEME_MASK         = 0x7,
 } cublasLtReductionScheme_t;
 
+/** Postprocessing options for the epilogue
+ */
+typedef enum {
+    /** No special postprocessing, just scale and quantize results if necessary.
+     */
+    CUBLASLT_EPILOGUE_DEFAULT = 1,
+
+    /** ReLu, apply ReLu point-wise transform to the results (x:=max(x, 0))
+     */
+    CUBLASLT_EPILOGUE_RELU = 2,
+
+    /** Bias, apply (broadcasted) Bias from bias vector. Bias vector length must match matrix D rows, it must be packed
+     * (stride between vector elements is 1). Bias vector is broadcasted to all columns and added before applying final
+     * postprocessing.
+     */
+    CUBLASLT_EPILOGUE_BIAS = 4,
+
+    /** ReLu and Bias, apply Bias and then ReLu transform
+     */
+    CUBLASLT_EPILOGUE_RELU_BIAS = (CUBLASLT_EPILOGUE_RELU | CUBLASLT_EPILOGUE_BIAS),
+} cublasLtEpilogue_t;
+
 /** Matmul heuristic search mode
  */
 typedef enum {
     /** ask heuristics for best algo for given usecase
      */
-    CUBLASLT_SEARCH_BEST_FIT           = 0,
+    CUBLASLT_SEARCH_BEST_FIT = 0,
     /** only try to find best config for preconfigured algo id
      */
     CUBLASLT_SEARCH_LIMITED_BY_ALGO_ID = 1,
@@ -614,9 +674,7 @@ typedef enum {
      */
     CUBLASLT_MATMUL_PREF_MATH_MODE_MASK,       
 
-    /** Reduction scheme mask, see cublasLtReductionScheme_t.
-     * 
-     * Only algorithm configurations specifying CUBLASLT_ALGO_CONFIG_REDUCTION_SCHEME that is not masked out by this attribte are allowed.
+    /** Reduction scheme mask, see cublasLtReductionScheme_t. Filters heuristic result to only include algo configs that use one of the required modes.
      * 
      * E.g. mask value of 0x03 will allow only INPLACE and COMPUTE_TYPE reduction schemes.
      * 
@@ -673,6 +731,19 @@ typedef enum {
      * float, default: 0.0f
      */
     CUBLASLT_MATMUL_PREF_MAX_WAVES_COUNT,
+
+    /** Pointer mode mask, see cublasLtPointerModeMask_t. Filters heuristic result to only include algorithms that support all required modes.
+     *
+     * uint32_t, default: (CUBLASLT_POINTER_MODE_MASK_HOST | CUBLASLT_POINTER_MODE_MASK_DEVICE) (only allows algorithms that support both regular host and device pointers)
+     */
+    CUBLASLT_MATMUL_PREF_POINTER_MODE_MASK,
+
+    /** Epilogue selector mask, see cublasLtEpilogue_t. Filters heuristic result to only include algorithms that support all required operations.
+     *
+     * uint32_t, default: CUBLASLT_EPILOGUE_DEFAULT (only allows algorithms that support default epilogue)
+     */
+    CUBLASLT_MATMUL_PREF_EPILOGUE_MASK,
+
 } cublasLtMatmulPreferenceAttributes_t;
 
 /** Create new matmul heuristic search preference descriptor.
@@ -911,16 +982,28 @@ typedef enum {
      * int32_t 0 means regular compute, 1 means tensor operations;
      */
     CUBLASLT_ALGO_CAP_MATHMODE_IMPL,
-    /** whether algorithm is implements gaussian optimization of complex matrix multiplication, see cublasMath_t
+    /** whether algorithm implements gaussian optimization of complex matrix multiplication, see cublasMath_t
      *
      * int32_t 0 means regular compute, 1 means gaussian;
      */
     CUBLASLT_ALGO_CAP_GAUSSIAN_IMPL,
-    /** whether algorithm is supports custom (not COL or ROW memory order), see cublasLtOrder_t
+    /** whether algorithm supports custom (not COL or ROW memory order), see cublasLtOrder_t
      *
      * int32_t 0 means only COL and ROW memory order is allowed, 1 means that algo might have different requirements;
      */
     CUBLASLT_ALGO_CAP_CUSTOM_MEMORY_ORDER,
+
+    /** bitmask enumerating pointer modes algorithm supports
+     *
+     * uint32_t, see cublasLtPointerModeMask_t
+     */
+    CUBLASLT_ALGO_CAP_POINTER_MODE_MASK,
+
+    /** bitmask enumerating kinds of postprocessing algorithm supports in the epilogue
+     *
+     * uint32_t, see cublasLtEpilogue_t
+     */
+    CUBLASLT_ALGO_CAP_EPILOGUE_MASK,
 } cublasLtMatmulAlgoCapAttributes_t;
 
 /** Get algo capability attribute.
