@@ -1,4 +1,4 @@
- /* Copyright 2009-2018 NVIDIA Corporation.  All rights reserved. 
+ /* Copyright 2009-2019 NVIDIA Corporation.  All rights reserved. 
  // 
  // NOTICE TO LICENSEE: 
  // 
@@ -51,8 +51,6 @@
 
 #define NVJPEGAPI
 
-// Maximum number of channels nvjpeg decoder supports
-#define NVJPEG_MAX_COMPONENT 4
 
 #include "cuda_runtime_api.h"
 #include "library_types.h"
@@ -63,25 +61,34 @@
   extern "C" {
 #endif
 
+// Maximum number of channels nvjpeg decoder supports
+#define NVJPEG_MAX_COMPONENT 4
+
+// nvjpeg version information
+#define NVJPEG_VER_MAJOR 10
+#define NVJPEG_VER_MINOR 2
+#define NVJPEG_VER_PATCH 0
+#define NVJPEG_VER_BUILD 0
+
 /* nvJPEG status enums, returned by nvJPEG API */
 typedef enum
 {
-    NVJPEG_STATUS_SUCCESS            = 0,
-    NVJPEG_STATUS_NOT_INITIALIZED    = 1,
-    NVJPEG_STATUS_INVALID_PARAMETER  = 2,
-    NVJPEG_STATUS_BAD_JPEG           = 3,
-    NVJPEG_STATUS_JPEG_NOT_SUPPORTED = 4,
-    NVJPEG_STATUS_ALLOCATOR_FAILURE  = 5,
-    NVJPEG_STATUS_EXECUTION_FAILED   = 6,
-    NVJPEG_STATUS_ARCH_MISMATCH      = 7,
-    NVJPEG_STATUS_INTERNAL_ERROR     = 8,
+    NVJPEG_STATUS_SUCCESS                       = 0,
+    NVJPEG_STATUS_NOT_INITIALIZED               = 1,
+    NVJPEG_STATUS_INVALID_PARAMETER             = 2,
+    NVJPEG_STATUS_BAD_JPEG                      = 3,
+    NVJPEG_STATUS_JPEG_NOT_SUPPORTED            = 4,
+    NVJPEG_STATUS_ALLOCATOR_FAILURE             = 5,
+    NVJPEG_STATUS_EXECUTION_FAILED              = 6,
+    NVJPEG_STATUS_ARCH_MISMATCH                 = 7,
+    NVJPEG_STATUS_INTERNAL_ERROR                = 8,
+    NVJPEG_STATUS_IMPLEMENTATION_NOT_SUPPORTED  = 9,
 } nvjpegStatus_t;
 
 
-// Enumeration returned by getImageInfo identifies image chroma subsampling stored inside JPEG input stream
+// Enum identifies image chroma subsampling values stored inside JPEG input stream
 // In the case of NVJPEG_CSS_GRAY only 1 luminance channel is encoded in JPEG input stream
 // Otherwise both chroma planes are present
-// Initial release support: 4:4:4, 4:2:0, 4:2:2, Grayscale
 typedef enum
 {
     NVJPEG_CSS_444 = 0,
@@ -94,24 +101,29 @@ typedef enum
     NVJPEG_CSS_UNKNOWN = -1
 } nvjpegChromaSubsampling_t;
 
-// Number of supported chroma subsamplings aliases
-#define NVJPEG_CSS_NUM 7
-
 // Parameter of this type specifies what type of output user wants for image decoding
 typedef enum
 {
-    NVJPEG_OUTPUT_UNCHANGED   = 0, // return decoded image as it is - write planar output
-    NVJPEG_OUTPUT_YUV         = 1, // return planar luma and chroma
-    NVJPEG_OUTPUT_Y           = 2, // return luma component only, write to 1-st channel of nvjpegImage_t
-    NVJPEG_OUTPUT_RGB         = 3, // convert to planar RGB
-    NVJPEG_OUTPUT_BGR         = 4, // convert to planar BGR
-    NVJPEG_OUTPUT_RGBI        = 5, // convert to interleaved RGB and write to 1-st channel of nvjpegImage_t
-    NVJPEG_OUTPUT_BGRI        = 6  // convert to interleaved BGR and write to 1-st channel of nvjpegImage_t
+    // return decompressed image as it is - write planar output
+    NVJPEG_OUTPUT_UNCHANGED   = 0,
+    // return planar luma and chroma, assuming YCbCr colorspace
+    NVJPEG_OUTPUT_YUV         = 1, 
+    // return luma component only, if YCbCr colorspace, 
+    // or try to convert to grayscale,
+    // writes to 1-st channel of nvjpegImage_t
+    NVJPEG_OUTPUT_Y           = 2,
+    // convert to planar RGB 
+    NVJPEG_OUTPUT_RGB         = 3,
+    // convert to planar BGR
+    NVJPEG_OUTPUT_BGR         = 4, 
+    // convert to interleaved RGB and write to 1-st channel of nvjpegImage_t
+    NVJPEG_OUTPUT_RGBI        = 5, 
+    // convert to interleaved BGR and write to 1-st channel of nvjpegImage_t
+    NVJPEG_OUTPUT_BGRI        = 6,
+    // maximum allowed value
+    NVJPEG_OUTPUT_FORMAT_MAX  = 6  
 } nvjpegOutputFormat_t;
 
-// Number of output formats
-#define NVJPEG_OUTPUT_FMT_NUM 7
-
 // Parameter of this type specifies what type of input user provides for encoding
 typedef enum
 {
@@ -122,21 +134,20 @@ typedef enum
 } nvjpegInputFormat_t;
 
 // Implementation
-// NVJPEG_BACKEND_DEFAULT    : defaults to  NVJPEG_BACKEND_GPU_HYBRID
+// NVJPEG_BACKEND_DEFAULT    : default value
 // NVJPEG_BACKEND_HYBRID     : uses CPU for Huffman decode
-// NVJPEG_BACKEND_GPU_HYBRID : nvjpegDecodeBatched will use GPU decoding for baseline JPEG images with interleaved scan when batch size is bigger than 100,
-//                             it uses CPU for other JPEG types. Other decode APIs will continue to use CPU for Huffman decode
+// NVJPEG_BACKEND_GPU_HYBRID : nvjpegDecodeBatched will use GPU decoding for baseline JPEG images with 
+//                             interleaved scan when batch size is bigger than 100, batched multi phase APIs 
+//                             will use CPU Huffman decode. All Single Image APIs will use GPU assisted huffman decode
 //
 typedef enum 
 {
     NVJPEG_BACKEND_DEFAULT = 0,
     NVJPEG_BACKEND_HYBRID  = 1,
     NVJPEG_BACKEND_GPU_HYBRID = 2,
-
 } nvjpegBackend_t;
 
-// Number of backends
-#define NVJPEG_BACKEND_NUM 3
+#define NVJPEG_FLAGS_DEFAULT 0
 
 // Output descriptor.
 // Data that is written to planes depends on output forman
@@ -184,6 +195,9 @@ typedef struct nvjpegJpegState* nvjpegJpegState_t;
 
 // returns library's property values, such as MAJOR_VERSION, MINOR_VERSION or PATCH_LEVEL
 nvjpegStatus_t NVJPEGAPI nvjpegGetProperty(libraryPropertyType type, int *value);
+// returns CUDA Toolkit property values that was used for building library, 
+// such as MAJOR_VERSION, MINOR_VERSION or PATCH_LEVEL
+nvjpegStatus_t NVJPEGAPI nvjpegGetCudartProperty(libraryPropertyType type, int *value);
 
 // Initalization of nvjpeg handle. This handle is used for all consecutive calls
 // IN         backend       : Backend to use. Currently Default or Hybrid (which is the same at the moment) is supported.
@@ -202,10 +216,10 @@ nvjpegStatus_t NVJPEGAPI nvjpegCreateSimple(nvjpegHandle_t *handle);
 // IN         flags         : Parameters for the operation. Must be 0.
 // INT/OUT    handle        : Codec instance, use for other calls
 nvjpegStatus_t NVJPEGAPI nvjpegCreateEx(nvjpegBackend_t backend, 
-          nvjpegDevAllocator_t *dev_allocator, 
-          nvjpegPinnedAllocator_t *pinned_allocator, 
-          unsigned int flags,
-          nvjpegHandle_t *handle);
+        nvjpegDevAllocator_t *dev_allocator, 
+        nvjpegPinnedAllocator_t *pinned_allocator, 
+        unsigned int flags,
+        nvjpegHandle_t *handle);
 
 // Release the handle and resources.
 // IN/OUT     handle: instance handle to release 
@@ -256,13 +270,13 @@ nvjpegStatus_t NVJPEGAPI nvjpegJpegStateDestroy(nvjpegJpegState_t jpeg_handle);
 // OUT        widths      : pointer to NVJPEG_MAX_COMPONENT of ints, returns width of each channel. 0 if channel is not encoded  
 // OUT        heights     : pointer to NVJPEG_MAX_COMPONENT of ints, returns height of each channel. 0 if channel is not encoded 
 nvjpegStatus_t NVJPEGAPI nvjpegGetImageInfo(
-          nvjpegHandle_t handle,
-          const unsigned char *data, 
-          size_t length,
-          int *nComponents, 
-          nvjpegChromaSubsampling_t *subsampling,
-          int *widths,
-          int *heights);
+        nvjpegHandle_t handle,
+        const unsigned char *data, 
+        size_t length,
+        int *nComponents, 
+        nvjpegChromaSubsampling_t *subsampling,
+        int *widths,
+        int *heights);
                    
 
 // Decodes single image. Destination buffers should be large enough to be able to store 
@@ -280,13 +294,13 @@ nvjpegStatus_t NVJPEGAPI nvjpegGetImageInfo(
 // 
 // \return NVJPEG_STATUS_SUCCESS if successful
 nvjpegStatus_t NVJPEGAPI nvjpegDecode(
-          nvjpegHandle_t handle,
-          nvjpegJpegState_t jpeg_handle,
-          const unsigned char *data,
-          size_t length, 
-          nvjpegOutputFormat_t output_format,
-          nvjpegImage_t *destination,
-          cudaStream_t stream);
+        nvjpegHandle_t handle,
+        nvjpegJpegState_t jpeg_handle,
+        const unsigned char *data,
+        size_t length, 
+        nvjpegOutputFormat_t output_format,
+        nvjpegImage_t *destination,
+        cudaStream_t stream);
 
 // Same functionality and parameters as for nvjpegDecodePlanar, but separated in steps: 
 // 1) CPU processing
@@ -384,7 +398,6 @@ nvjpegStatus_t NVJPEGAPI nvjpegDecodeBatchedPhaseThree(
           nvjpegImage_t *destinations,
           cudaStream_t stream);
 
-
 /**********************************************************
 *                        Compression                      *
 **********************************************************/
@@ -458,6 +471,178 @@ nvjpegStatus_t NVJPEGAPI nvjpegEncodeRetrieveBitstream(
         cudaStream_t stream);
 
 
+///////////////////////////////////////////////////////////////////////////////////
+// API v2 //
+///////////////////////////////////////////////////////////////////////////////////
+
+
+///////////////////////////////////////////////////////////////////////////////////
+// NVJPEG buffers //
+///////////////////////////////////////////////////////////////////////////////////
+
+struct nvjpegBufferPinned;
+typedef struct nvjpegBufferPinned* nvjpegBufferPinned_t;
+
+nvjpegStatus_t NVJPEGAPI nvjpegBufferPinnedCreate(nvjpegHandle_t handle, 
+    nvjpegPinnedAllocator_t* pinned_allocator,
+    nvjpegBufferPinned_t* buffer);
+
+nvjpegStatus_t NVJPEGAPI nvjpegBufferPinnedDestroy(nvjpegBufferPinned_t buffer);
+
+struct nvjpegBufferDevice;
+typedef struct nvjpegBufferDevice* nvjpegBufferDevice_t;
+
+nvjpegStatus_t NVJPEGAPI nvjpegBufferDeviceCreate(nvjpegHandle_t handle, 
+    nvjpegDevAllocator_t* device_allocator, 
+    nvjpegBufferDevice_t* buffer);
+
+nvjpegStatus_t NVJPEGAPI nvjpegBufferDeviceDestroy(nvjpegBufferDevice_t buffer);
+
+// retrieve buffer size and pointer - this allows reusing buffer when decode is not needed
+nvjpegStatus_t NVJPEGAPI nvjpegBufferPinnedRetrieve(nvjpegBufferPinned_t buffer, size_t* size, void** ptr);
+
+nvjpegStatus_t NVJPEGAPI nvjpegBufferDeviceRetrieve(nvjpegBufferDevice_t buffer, size_t* size, void** ptr);
+
+// this allows attaching same memory buffers to different states, allowing to switch implementations
+// without allocating extra memory
+nvjpegStatus_t NVJPEGAPI nvjpegStateAttachPinnedBuffer(nvjpegJpegState_t decoder_state,
+    nvjpegBufferPinned_t pinned_buffer);
+
+nvjpegStatus_t NVJPEGAPI nvjpegStateAttachDeviceBuffer(nvjpegJpegState_t decoder_state,
+    nvjpegBufferDevice_t device_buffer);
+
+///////////////////////////////////////////////////////////////////////////////////
+// JPEG stream parameters //
+///////////////////////////////////////////////////////////////////////////////////
+
+// handle that stores stream information - metadata, encoded image parameters, encoded stream parameters
+// stores everything on CPU side. This allows us parse header separately from implementation
+// and retrieve more information on the stream. Also can be used for transcoding and transfering 
+// metadata to encoder
+struct nvjpegJpegStream;
+typedef struct nvjpegJpegStream* nvjpegJpegStream_t;
+
+nvjpegStatus_t NVJPEGAPI nvjpegJpegStreamCreate(
+    nvjpegHandle_t handle, 
+    nvjpegJpegStream_t *jpeg_stream);
+
+nvjpegStatus_t NVJPEGAPI nvjpegJpegStreamDestroy(nvjpegJpegStream_t jpeg_stream);
+
+nvjpegStatus_t NVJPEGAPI nvjpegJpegStreamParse(
+    nvjpegHandle_t handle,
+    const unsigned char *data, 
+    size_t length,
+    int save_metadata,
+    int save_stream,
+    nvjpegJpegStream_t jpeg_stream);
+
+nvjpegStatus_t NVJPEGAPI nvjpegJpegStreamGetFrameDimensions(
+    nvjpegJpegStream_t jpeg_stream,
+    unsigned int* width,
+    unsigned int* height);
+
+nvjpegStatus_t NVJPEGAPI nvjpegJpegStreamGetComponentsNum(
+    nvjpegJpegStream_t jpeg_stream,
+    unsigned int* components_num);
+
+nvjpegStatus_t NVJPEGAPI nvjpegJpegStreamGetComponentDimensions(
+    nvjpegJpegStream_t jpeg_stream,
+    unsigned int component,
+    unsigned int* width,
+    unsigned int* height);
+
+// if encoded is 1 color component then it assumes 4:0:0 (NVJPEG_CSS_GRAY, grayscale)
+// if encoded is 3 color components it tries to assign one of the known subsamplings
+//   based on the components subsampling infromation
+// in case sampling factors are not stadard or number of components is different 
+//   it will return NVJPEG_CSS_UNKNOWN
+nvjpegStatus_t NVJPEGAPI nvjpegJpegStreamGetChromaSubsampling(
+    nvjpegJpegStream_t jpeg_stream,
+    nvjpegChromaSubsampling_t* chroma_subsampling);
+
+///////////////////////////////////////////////////////////////////////////////////
+// Decode parameters //
+///////////////////////////////////////////////////////////////////////////////////
+// decode parameters structure. Used to set decode-related tweaks
+struct nvjpegDecodeParams;
+typedef struct nvjpegDecodeParams* nvjpegDecodeParams_t;
+
+nvjpegStatus_t NVJPEGAPI nvjpegDecodeParamsCreate(
+    nvjpegHandle_t handle, 
+    nvjpegDecodeParams_t *decode_params);
+
+nvjpegStatus_t NVJPEGAPI nvjpegDecodeParamsDestroy(nvjpegDecodeParams_t decode_params);
+
+// set output pixel format - same value as in nvjpegDecode()
+nvjpegStatus_t NVJPEGAPI nvjpegDecodeParamsSetOutputFormat(
+    nvjpegDecodeParams_t decode_params,
+    nvjpegOutputFormat_t output_format);
+
+// set to desired ROI. set to (0, 0, -1, -1) to disable ROI decode (decode whole image)
+nvjpegStatus_t NVJPEGAPI nvjpegDecodeParamsSetROI(
+    nvjpegDecodeParams_t decode_params,
+    int offset_x, int offset_y, int roi_width, int roi_height);
+
+// set to true to allow conversion from CMYK to RGB or YUV that follows simple subtractive scheme
+nvjpegStatus_t NVJPEGAPI nvjpegDecodeParamsSetAllowCMYK(
+    nvjpegDecodeParams_t decode_params,
+    int allow_cmyk);
+
+///////////////////////////////////////////////////////////////////////////////////
+// Decoder helper functions //
+///////////////////////////////////////////////////////////////////////////////////
+
+struct nvjpegJpegDecoder;
+typedef struct nvjpegJpegDecoder* nvjpegJpegDecoder_t;
+
+//creates decoder implementation
+nvjpegStatus_t NVJPEGAPI nvjpegDecoderCreate(nvjpegHandle_t nvjpeg_handle, 
+    nvjpegBackend_t implementation, 
+    nvjpegJpegDecoder_t* decoder_handle);
+
+nvjpegStatus_t NVJPEGAPI nvjpegDecoderDestroy(nvjpegJpegDecoder_t decoder_handle);
+
+// on return sets is_supported value to 0 if decoder is capable to handle jpeg_stream 
+// with specified decode parameters
+nvjpegStatus_t NVJPEGAPI nvjpegDecoderJpegSupported(nvjpegJpegDecoder_t decoder_handle, 
+    nvjpegJpegStream_t jpeg_stream,
+    nvjpegDecodeParams_t decode_params,
+    int* is_supported);
+
+// creates decoder state 
+nvjpegStatus_t NVJPEGAPI nvjpegDecoderStateCreate(nvjpegHandle_t nvjpeg_handle,
+    nvjpegJpegDecoder_t decoder_handle,
+    nvjpegJpegState_t* decoder_state);
+
+///////////////////////////////////////////////////////////////////////////////////
+// Decode functions //
+///////////////////////////////////////////////////////////////////////////////////
+// starts decoding on host and save decode parameters to the state
+nvjpegStatus_t NVJPEGAPI nvjpegDecodeJpegHost(
+    nvjpegHandle_t handle,
+    nvjpegJpegDecoder_t decoder,
+    nvjpegJpegState_t decoder_state,
+    nvjpegDecodeParams_t decode_params,
+    nvjpegJpegStream_t jpeg_stream);
+
+// hybrid stage of decoding image,  involves device async calls
+// note that jpeg stream is a parameter here - because we still might need copy 
+// parts of bytestream to device
+nvjpegStatus_t NVJPEGAPI nvjpegDecodeJpegTransferToDevice(
+    nvjpegHandle_t handle,
+    nvjpegJpegDecoder_t decoder,
+    nvjpegJpegState_t decoder_state,
+    nvjpegJpegStream_t jpeg_stream,
+    cudaStream_t stream);
+
+// finishing async operations on the device
+nvjpegStatus_t NVJPEGAPI nvjpegDecodeJpegDevice(
+    nvjpegHandle_t handle,
+    nvjpegJpegDecoder_t decoder,
+    nvjpegJpegState_t decoder_state,
+    nvjpegImage_t *destination,
+    cudaStream_t stream);
+
 #if defined(__cplusplus)
   }
 #endif
